/**
 * Provides transfer object classes to serialize/deserialize the 
 * application configuration.
 * 
 * <p>Most classes in this package use the Simple framework for XML
 * serialization, and its corresponding annotation-based configuration.
 *
 * @see <a href="http://simple.sourceforge.net/">Simple framework</a> 
 */
package org.syncany.config.to;