/**
 * Provides classes necessary for the application update and
 * version check against the Syncany API.
 * 
 * <p>The central class in this package is the 
 * {@link org.syncany.operations.update.UpdateOperation UpdateOperation}. 
 * It is executed by the command line client and/or via the daemon API.
 * 
 * @see <a href="https://github.com/syncany/syncany-website">Syncany Website/API</a>
 * @see org.syncany.operations.Operation
 */
package org.syncany.operations.update;