/**
 * Provides classes for encryption/decryption of files and byte
 * streams. It implements the application's crypto concept.
 * 
 * @see <a href="http://syncany.readthedocs.org/en/latest/security.html">Syncany security concepts</a>
 */
package org.syncany.crypto;