DELETE FROM multichunk_muddy;
DELETE FROM multichunk_chunk;
DELETE FROM multichunk;
DELETE FROM fileversion;
DELETE FROM filehistory;
DELETE FROM filecontent_chunk;
DELETE FROM filecontent;
DELETE FROM databaseversion_vectorclock;
DELETE FROM chunk;
DELETE FROM databaseversion;
DELETE FROM known_databases;
